
from graph.dataset import Dataset
from utils import calendarDate

import numpy

import ccgfilt

#----------------------------------------------
def createDataset(graph, x, y, name, color, symbol, linewidth=1):
	""" Create a graph dataset and add it to the graph """

	if len(x) == 0 or len(y) == 0:
		return False

	dataset = graph.getDataset(name)
	if dataset is None:
		dataset = Dataset(x, y, name)
		dataset.style.setLineColor(color)
		dataset.style.setMarker(symbol)
		dataset.style.setFillMarkers(0)
		dataset.style.setLineWidth(linewidth)
		graph.addDataset(dataset)
	else:
		dataset.SetData(x, y)

	return True

#----------------------------------------------
def processData(grapher, x, y, name):
	""" Filter the data, create datasets from data in x, y """

	grapher.SetStatusText("Processing Data...")

	# needed for export dialog
	date = x[0]
	yr, m, d, h, n, s = calendarDate(date)
	grapher.startyear = yr
	grapher.startmon = m
	grapher.startday = d

	gap = 0
	if grapher.parameters.use_gap:
		gap = grapher.parameters.gapsize
	filt = ccgfilt.ccgFilter(x, y, grapher.parameters.short, grapher.parameters.long, grapher.parameters.interval, grapher.parameters.npoly, grapher.parameters.nharm, grapher.parameters.zero, gap, use_gain_factor=grapher.parameters.use_gain, debug=False)

	mm = filt.getMonthlyMeans()
	annm = filt.getAnnualMeans()
	amps = filt.getAmplitudes()
	tcup, tcdown = filt.getTrendCrossingDates()

	grapher.fstats = filt.stats()
	grapher.mmcstats = getMonthlyStats(mm)
	grapher.annstats = getAnnualStats(annm)
	grapher.tcstats = getTrendCrossing(tcup, tcdown)
	grapher.scstats = getAmplitudeStats(amps)

	# Seasonal cycle amplitudes ----------------------------
	a = numpy.array(amps)

	# columns are year, total amplitude, date of max amplitude, max amplitude, date of min amplitude, min amplitude
	page = grapher.nb.GetPage(4)
	if a.size > 0:
		createDataset(page, a[:, 0]+0.5, a[:, 1], 'Amplitude', 'blue', 'circle', linewidth=2)
		createDataset(page, a[:, 2], a[:, 3], 'Amplitude Max', 'red', 'circle', 2)
		createDataset(page, a[:, 4], a[:, 5], 'Amplitude Min', 'green', 'circle', 2)
		page.title.text = name
		page.update()


        # Data plot -------------------------
	x0 = filt.xinterp
	y1 = filt.getFunctionValue(x0)
	y2 = filt.getPolyValue(x0)
	y3 = filt.getSmoothValue(x0)
	y4 = filt.getTrendValue(x0)

	page = grapher.nb.GetPage(0)
	createDataset(page, x, y, 'Data', 'black', 'square')
	createDataset(page, x0, y1, 'Function', 'magenta', 'None', linewidth=2)
	createDataset(page, x0, y2, 'Poly', 'cyan', 'None', linewidth=2)
	createDataset(page, x0, y3, 'Smoothed', 'red', 'None', linewidth=2)
	createDataset(page, x0, y4, 'Trend', 'blue', 'None', linewidth=2)
	page.title.text = name
	page.update()


        # Seasonal Cycle plot ------------------------
	trend = filt.getTrendValue(x)
	ys2 = y - trend
	y6 = filt.getHarmonicValue(x0)
	y5 = y6 + filt.smooth - filt.trend

	page = grapher.nb.GetPage(1)
	createDataset(page, x, ys2, 'Detrended data', 'black', 'square')
	createDataset(page, x0, y5, 'Smoothed Cycle', 'red', 'None', linewidth=2)
	createDataset(page, x0, y6, 'Harmonics', 'blue', 'None', linewidth=2)
	page.title.text = name
	page.update()

        # Residuals plot -----------------------
	ys3 = filt.resid
	yy = filt.getSmoothValue(x)
	ys4 = filt.yp - yy
	y7 = filt.smooth
	y8 = filt.trend

	x1 = filt.xinterp
	y9 = filt.yinterp

	page = grapher.nb.GetPage(2)
	createDataset(page, x, ys3, 'Residuals', 'black', 'square')
	createDataset(page, x, ys4, 'Residuals from Smooth', 'black', 'plus')
	createDataset(page, x1, y9, 'Interpolated/Equal spaced data', '#00bb00', 'circle')
	createDataset(page, x0, y7, 'Smoothed Residuals', 'red', 'None', linewidth=2)
	createDataset(page, x0, y8, 'Trend Residuals', 'blue', 'None', linewidth=2)
	page.title.text = name
	page.update()

        # Growth Rate plot ----------------------------
	y9 = filt.deriv

	page = grapher.nb.GetPage(3)
	createDataset(page, x0, y9, 'Growth Rate', 'red', 'None', linewidth=2)
	page.title.text = name
	page.update()

	# Filter response plot ------------------------
	page = grapher.nb.GetPage(5)

	freq, r = filt.getFilterResponse(grapher.parameters.short)
	createDataset(page, freq, r, 'Short Term Filter Response', 'red', 'None', linewidth=2)
	freq, r = filt.getFilterResponse(grapher.parameters.long)
	createDataset(page, freq, r, 'Long Term Filter Response', 'blue', 'None', linewidth=2)
	page.update()


	grapher.update_menus(1)
	grapher.SetStatusText("")


#----------------------------------------------
def getAmplitudeStats(amps):

	months = ["", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]


	s = """
*****  Season Cycle Statistics.  *****

 Year      Amplitude     Maximum   Date     Minimum   Date
-----------------------------------------------------------
"""

	for (year, amp, maxdate, maxval, mindate, minval) in amps:

		(yr, mnmax, dmax, hr, mn, sec) = calendarDate(maxdate)
		(yr, mnmin, dmin, hr, mn, sec) = calendarDate(mindate)

		s += "%5.0f %12.2f %12.2f   %3s %2d %9.2f   %3s %2d\n" % (year, amp, maxval, months[mnmax], dmax, minval, months[mnmin], dmin)

	return s

#----------------------------------------------
def getMonthlyStats(mm):

	s = ""
	for (year, month, val, std, n) in mm:

		s += "%4d %02d %7.2f %5.2f %2d\n" % (year, month, val, std, n)

	return s


#----------------------------------------------
def getAnnualStats(ann):

	s = ""
	for (year, val, std, n) in ann:

		s += "%4d %7.2f %5.2f %2d\n" % (year, val, std, n)

	return s

#----------------------------------------------
def getTrendCrossing(tcup, tcdown):

	months = ["", "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]

	s = "Trend Crossing Dates\n    - to +           + to -\n" + "-"*40 + "\n"

	for up, down in zip(tcup, tcdown):

		(uyr, umon, uday, hr, mn, sec) = calendarDate(up)
		(dyr, dmon, dday, hr, mn, sec) = calendarDate(down)

		s += "    %02d %s %4d    %02d %s %4d\n" % (uday, months[umon], uyr, dday, months[dmon], dyr)


	return s
